import asyncio
from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor
from common import logger

log = logger.getUnloadLogger('main')


async def run_parallel_tasks(tasks, max_workers=4):
    queue = asyncio.Queue()

    for task in tasks:
        await queue.put(task)

    async def worker():
        # тут можно использовать ProcessPoolExecutor или ThreadPoolExecutor
        # ProcessPoolExecutor работает быстрее, но с использованием fork-процессов, ThreadPoolExecutor работает медленнее, но мб стабильней
        with ThreadPoolExecutor(max_workers=max_workers) as executor:
            while not queue.empty():
                task_id, fn, callback, args = await queue.get()
                try:
                    result = await asyncio.get_event_loop().run_in_executor(executor, fn, args['shop'],
                                                                            args['work_directory'], args['cash'],
                                                                            args['task'],
                                                                            args.get('file_flag', None),
                                                                            args.get('converterName', None),
                                                                            args.get('closedShifts', 0),
                                                                            args.get('type', None))
                    if callback:
                        callback(result)
                except Exception as e:
                    log.info(f"Task failed: {e}")
                finally:
                    queue.task_done()
            executor.shutdown(wait=False, cancel_futures=True)

    workers = [asyncio.create_task(worker()) for _ in range(max_workers)]
    await asyncio.gather(*workers)


async def run_parallel(tasks, max_workers):
    await run_parallel_tasks(tasks, max_workers=max_workers)


def run(tasks, max_workers):
    asyncio.run(run_parallel(tasks, max_workers))
